#
# pushare - P2P file sharing system
#
# Copyright (c) 2004 Satoshi Fukutomi <fuktommy@inter7.jp>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the University nor the names of its contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: Cache.pm,v 1.2 2004/06/05 02:04:07 fuktommy Exp $
#

package PuShare::Cache;
use strict;
use IPC::Open2;
use Jcode;
use PuShare::Config;

my $cachedir = $PuShare::Config::cachedir;
my $updir    = $PuShare::Config::updir;
my $downdir  = $PuShare::Config::downdir;
my $namelist = $PuShare::Config::namelist;

sub list(;%) {
	my %opt  = @_;
	my %name = ();
	local $_;
	PuShare::Util::lock() if ($opt{lock});
	if (open IN, $namelist) {
		while (<IN>) {
			chomp;
			local @_ = split /<>/, $_, 2;
			$name{$_[0]} = $_[1];
		}
		close IN;
	}
	PuShare::Util::unlock() if ($opt{lock});
	return %name;
}

sub update() {
	PuShare::Util::lock();
	local $_;
	my %name = list();
	open OUT, "> $namelist" or die "$namelist: open failed. $!";
	my %cache = reverse(%name);
	while (glob ("$updir/*")) {
		my $file = $_;
		$file =~ s|$updir/||;
		$file = Jcode->new($file)->h2z->utf8;
		$file =~ s|[\\/]||g;
		unless ($cache{$file}) {
			my($IN, $OUT);
			my $pid = open2 $IN, $OUT, $PuShare::Config::command{md5sum}, $_;
			close $OUT;
			my $md5 = <$IN>;
			close $IN;
			wait;
			$md5 = substr $md5, 0, 32;
			system $PuShare::Config::command{cp}, $_, "$cachedir/$md5.dat";
			$name{$md5} = $file;
		}
	}
	my ($k, $v);
	while (($k, $v) = each %name) {
		if (-s "$cachedir/$k.dat") {
			print OUT "$k<>$v\n";
		}
	}
	close OUT;
	PuShare::Util::unlock();
}

sub delete($) {
	my($id) = @_;
	unlink "$cachedir/$id.dat" or return;
	PuShare::Util::lock();
	my %name = list();
	open OUT, "> $namelist" or die "$namelist: open failed. $!";
	my ($k, $v);
	while (($k, $v) = each %name) {
		if ($k ne $id) {
			print OUT "$k<>$v\n";
		}
	}
	close OUT;
	PuShare::Util::unlock();
}

sub add($$) {
	my($id, $name) = @_;
	$name =~ s|[\\/]||g;
	PuShare::Util::lock();
	open OUT, ">> $namelist" or die "$namelist: open failed. $!";
	print OUT "$id<>$name\n";
	close OUT;
	PuShare::Util::unlock();
}

sub name($) {
	my($id) = @_;
	my %name = list();
	return $name{$id};
}

sub convert($) {
	my($id) = @_;
	my $name = name($id);
	$name = "$id.dat" unless ($name);
	Jcode::convert(\$name, $PuShare::Config::jcode, "utf8");
	while (-e "$downdir/$name") {
		my @x = split /\./, $name;
		if ($x[-2] =~ /^\d+$/) {
			$x[-2]++;
		} else {
			my $tmp = $x[-1];
			$x[-1] = 1;
			push @x, $tmp;
		}
		$name = join ".", @x;
	}
	system $PuShare::Config::command{cp}, "$cachedir/$id.dat", "$downdir/$name";
}


1;
