#
# pushare - P2P file sharing system
#
# Copyright (c) 2004 Satoshi Fukutomi <fuktommy@inter7.jp>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the University nor the names of its contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: Node.pm,v 1.2 2004/06/05 02:04:07 fuktommy Exp $
#

package PuShare::Node;
use strict;
use IPC::Open2;
use Socket;
use PuShare::Config;

my $cachedir = $PuShare::Config::cachedir;

#
# constructor
# args: host, port
#
sub new($$$) {
	my($this, $host, $port) = @_;
	$this = { host => $host,
		  port => $port,
		};
	return bless $this;
}

#
# connect other node
# args: path, options
# options: file
#
sub talk($$@) {
	my($this, $path, %opt) = @_;
	my($host, $port) = ($$this{host}, $$this{port});
	local $_;
	my $timeout = $PuShare::Config::timeout;
	my $bufsize = $PuShare::Config::bufsize;
	#warn "$host:$port$path";
	my @buf = ();
	pipe IN, OUT;
	if (fork) {
		close OUT;
		local $_;
		local $SIG{CHLD} = "DEFAULT";
		my $entity = 0;
		while (<IN>) {
			if ((! $entity) && (/^[\r\n]*$/)) {
				$entity = 1;
				if ($opt{file}) {
					open OUT2, "> $cachedir/$opt{file}.dat" or die;
				}
			} elsif (! $entity) {
				next;
			} elsif ($opt{file}) {
				print OUT2;
			} else {
				push @buf, $_;
			}
		}
		close IN;
		wait;
		if ($opt{file}) {
			close OUT2;
			if (-s "$cachedir/$opt{file}.dat") {
				my($IN, $OUT);
				my $pid = open2 $IN, $OUT, $PuShare::Config::command{md5sum}, "$cachedir/$opt{file}.dat";
				close $OUT;
				my $md5 = <$IN>;
				close $IN;
				wait;
				$md5 = substr $md5, 0, 32;
				if ($md5 eq $opt{file}) {
					return (-s "$cachedir/$opt{file}.dat");
				}
			}
			if (-e "$cachedir/$opt{file}.dat") {
				unlink "$cachedir/$opt{file}.dat";
			}
			return 0;
		} else {
			s/[\r\n]//g foreach @buf;
			return @buf;
		}

	} else {
		close IN;
		my($addr, $sockaddr);
		my $try = $PuShare::Config::retry;
		local $SIG{ALRM} = sub { close OUT;
					 die "$host:$port: connection timeout."; };
		START:
		alarm $timeout;
		unless ($addr = inet_aton $host) {
			if ($try-- > 0) {
				sleep 3;
				goto START;
			}
			close OUT;
			die "$host: faild to resolv. $!";
		}
		$sockaddr = pack_sockaddr_in($port, $addr);
		unless (socket SOCKET, PF_INET, SOCK_STREAM, 0) {
			if ($try-- > 0) {
				sleep 3;
				goto START;
			}
			close OUT;
			die "failed to open socket. $!";
		}
		unless (connect SOCKET, $sockaddr) {
			if ($try-- > 0) {
				sleep 3;
				goto START;
			}
			close OUT;
			die "$host:$port: fault to connect. $!";
		}
		select SOCKET; $| = 1;
		local $, = "\r\n";
		local $_;
		alarm $timeout;
		my $ver = $PuShare::Config::version;
		print "GET $path HTTP/1.0",
		      "User-Agent: pushare/$ver",
		      "Host: $host:$port",
		      "Accept: text/plain",
		      "Connection: close",
		      "\r\n";
		my $buf;
		while (read SOCKET, $buf, $bufsize) {
			print OUT $buf;
			alarm $timeout;
		}
		alarm 0;
		close SOCKET;
		close OUT;
		exit;
	}
}

#
# to string
#
sub toString($) {
	my($this) = @_;
	my($host, $port) = ($$this{host}, $$this{port});
	return "$host:$port";
}

#
# ping node
#
sub ping($) {
	my($this) = @_;
	my @buf = $this->talk("/ping");
	return ((defined $buf[0])&&($buf[0] eq "PONG"));
}

#
# connect
#
sub join($) {
	my($this) = @_;
	my $port = $main::port;
	my @buf = $this->talk("/join?port=$port");
	return @buf;
}

#
# unconnect
#
sub bye($) {
	my($this) = @_;
	my $port = $main::port;
	my @buf = $this->talk("/bye?port=$port");
	return (@buf > 0) && ($buf[0] eq "BYEBYE");
}

1;
